from typing import Union


def _convert_data(data: Union[str, bytes],
                  encoding='utf-8') -> bytes:
    if isinstance(data, str):
        data = data.encode(encoding=encoding)
    return data



import logging
import time
import traceback
import datetime
import json
from typing import Optional


class LoguruSerializeAdapterFormatter(logging.Formatter):

    def __init__(self, fmt=None, datefmt=None, style='%', validate=True, extra: Optional[dict] = None):
        super().__init__(fmt, datefmt, style, validate)
        self._extra = extra


    def format(self, record):
        elapsed_seconds = time.time() - record.created
        elapsed_repr = self._format_elapsed(elapsed_seconds)
        exception_msg = self._format_line_break(traceback.format_exc()) if record.exc_info else ''
        message = self._format_line_break(record.getMessage())
        log_record = {
            "text": message + f"\n{exception_msg}",
            "record": {
                "elapsed": {
                    "repr": elapsed_repr,
                    "seconds": elapsed_seconds
                },
                "exception": self._format_exception(record.exc_info),
                "extra": self._extra or {},
                "file": {
                    "name": record.filename,
                    "path": record.pathname
                },
                "function": record.funcName,
                "level": {
                    "icon": self._get_level_icon(record.levelname),
                    "name": record.levelname,
                    "no": record.levelno
                },
                "line": record.lineno,
                "message": message,
                "module": record.module,
                "name": record.name,
                "process": {
                    "id": record.process,
                    "name": record.processName
                },
                "thread": {
                    "id": record.thread,
                    "name": record.threadName
                },
                "time": {
                    "repr": datetime.datetime.fromtimestamp(record.created).strftime('%Y-%m-%d %H:%M:%S.%f%z'),
                    "timestamp": record.created
                }
            }
        }
        return json.dumps(log_record, ensure_ascii=False)

    @staticmethod
    def _get_level_icon(level_name: str):
        if level_name == 'TRACE':
            return '✏️'
        if level_name == 'DEBUG':
            return '🐞'
        elif level_name == 'INFO':
            return 'ℹ️'
        elif level_name == 'WARNING':
            return '⚠️'
        elif level_name == 'ERROR':
            return '❌'
        elif level_name == 'CRITICAL':
            return '☠️'
        return ''

    @staticmethod
    def _format_elapsed(elapsed_seconds):
        hours, remainder = divmod(elapsed_seconds, 3600)
        minutes, seconds = divmod(remainder, 60)
        return "{:0>2}:{:0>2}:{:0>2}.{:06}".format(int(hours), int(minutes), int(seconds),
                                                   int((elapsed_seconds - int(elapsed_seconds)) * 1000000))

    def _format_exception(self, exc_info):
        if exc_info:
            exc_type, exc_value, exc_traceback = exc_info
            return {
                "type": exc_type.__name__,
                "message": self._format_line_break(str(exc_value)),
                "traceback": True
            }
        else:
            return None

    @staticmethod
    def _format_line_break(text: str) -> str:
        # return text.replace('\r', '\\r').replace('\n', '\\n')
        return text


def get_loguru_adapter_logging_formatter():
    """获取适配loguru serialize格式的logging.Formatter类型

    Returns:
        返回logging.Formatter类
    """
    return LoguruSerializeAdapterFormatter



import os
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.utils import formataddr
from email import encoders
from typing import Union, List, Tuple


def send_email(smtp_server: str,
               smtp_port: int,
               from_email: str,
               from_password: str,
               title: str,
               body: str,
               to_emails: Union[List[str], Tuple[str], str],
               from_name: str = None,
               ssl=True,
               to_cc_mails: Union[List[str], Tuple[str], str, None] = None,
               attachments: Union[List[str], Tuple[str], str, None] = None):
    """发送邮件功能, 支持抄送、附件, 目前仅支持 html 渲染的 body 样式

    Args:
        smtp_server: SMTP 服务器域名, 例如: smtp.feishu.cn
        smtp_port: SMTP 服务器端口, 例如: 465
        from_email: 发件人邮箱, 例如: xxx@xxx.com
        from_password: 发件人密码
        from_name: 发件人名称
        ssl: 是否使用SMTP_SSL连接服务器
        title: 邮件主题
        body: 邮件正文
        to_emails: 收件人, 支持多个收件人
        to_cc_mails: 抄送人, 支持多个抄送人
        attachments: 附件, 支持多个附件, 附件的绝对路径或相对路径

    Raises:
        SMTPException: 发送邮件异常
    """
    message = MIMEMultipart()
    # 主题
    message['Subject'] = Header(title, 'utf-8')
    # 发件人
    message['From'] = formataddr(
        (from_name, from_email), 'utf-8')
    # 抄送人
    if isinstance(to_cc_mails, (tuple, list)):
        to_cc_mails = ','.join(to_cc_mails)
    message['Cc'] = to_cc_mails
    # 邮件正文
    message.attach(MIMEText(body, 'html'))
    # 附件
    if attachments:
        if isinstance(attachments, str):
            attachments = [attachments]
        for attachment in attachments:
            with open(attachment, 'rb') as f:
                part = MIMEBase('application', 'octet-stream')
                part.set_payload(f.read())
            encoders.encode_base64(part)
            filename = attachment.split(os.sep)[-1]
            part.add_header('Content-Disposition',
                            f'attachment; filename= {filename}')
            message.attach(part)

    # 连接服务器发送邮件
    if ssl:
        server = smtplib.SMTP_SSL(smtp_server, smtp_port)
    else:
        server = smtplib.SMTP(smtp_server, smtp_port)
    server.login(from_email, from_password)
    server.send_message(message, from_email, to_emails)


class SmtpEmailNotice:
    """
    邮件通知工具
    """

    def __init__(self,
                 smtp_server: str,
                 smtp_port: int,
                 from_email: str,
                 from_password: str,
                 from_name: str = None,
                 ssl=True, ):
        """
        Args:
            smtp_server: SMTP 服务器域名, 例如: smtp.feishu.cn
            smtp_port: SMTP 服务器端口, 例如: 465
            from_email: 发件人邮箱, 例如: xxx@xxx.com
            from_password: 发件人密码
            from_name: 发件人名称
            ssl: 是否使用SMTP_SSL连接服务器
        """
        self._smtp_server = smtp_server
        self._smtp_port = smtp_port
        self._from_email = from_email
        self._from_password = from_password
        self._from_name = from_name if from_name else from_email
        self._ssl = ssl

    def send_email(self,
                   title: str,
                   body: str,
                   to_emails: Union[List[str], Tuple[str], str],
                   to_cc_mails: Union[List[str], Tuple[str], str, None] = None,
                   attachments: Union[List[str], Tuple[str], str, None] = None):
        """发送邮件功能, 支持抄送、附件, 目前仅支持 html 渲染的 body 样式

        Args:
            title: 邮件主题
            body: 邮件正文
            to_emails: 收件人, 支持多个收件人
            to_cc_mails: 抄送人, 支持多个抄送人
            attachments: 附件, 支持多个附件, 附件的绝对路径或相对路径

        Raises:
            SMTPException: 发送邮件异常
        """
        return send_email(smtp_server=self._smtp_server,
                          smtp_port=self._smtp_port,
                          from_email=self._from_email,
                          from_password=self._from_password,
                          title=title,
                          body=body,
                          to_emails=to_emails,
                          from_name=self._from_name,
                          ssl=self._ssl,
                          to_cc_mails=to_cc_mails,
                          attachments=attachments)



from loguru import logger as log
from typing import Union, List

from .base import DataFileLoader, DataFileExtractor


class YamlDataFileLoader(DataFileLoader):

    def _convert_config_data_to_python(self,
                                       config_data: Union[str, bytes],
                                       **kwargs) -> Union[dict, tuple, list]:
        try:
            import yaml
            return yaml.safe_load(config_data)
        except ImportError as e:
            log.warning('未安装pyyaml模块, 无法解析yaml格式数据')
            raise e


class YamlDataFileExtractor(DataFileExtractor):

    def _convert_data_to_str(self, data: Union[dict, List[dict]],
                             **kwargs):
        try:
            import yaml
            return yaml.dump(data)
        except ImportError as e:
            log.warning('未安装pyyaml模块, 无法解析yaml格式数据')
            raise e



from abc import ABC, abstractmethod
from typing import Any


class BaseBuilder(ABC):
    """通用建造器父类, 所有建造器应继承该类"""

    @abstractmethod
    def build(self) -> Any:
        raise NotImplementedError()

    def __str__(self):
        return self.build()



import math


def ease_in_sine(x: float) -> float:
    """缓动函数easeInSine

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - math.cos((x * math.pi) / 2)


def ease_out_sine(x: float) -> float:
    """缓动函数easeOutSine

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return math.sin((x * math.pi) / 2)


def ease_in_out_sine(x: float) -> float:
    """缓动函数easeInOutSine

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return -(math.cos(math.pi * x) - 1) / 2


def ease_in_quad(x: float) -> float:
    """缓动函数easeInQuad

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return x * x


def ease_out_quad(x: float) -> float:
    """缓动函数easeOutQuad

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - (1 - x) * (1 - x)


def ease_in_out_quad(x: float) -> float:
    """缓动函数easeInOutQuad

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 2 * x * x if x < 0.5 else 1 - math.pow(-2 * x + 2, 2) / 2


def ease_in_cubic(x: float) -> float:
    """缓动函数easeInCubic

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return math.pow(x, 3)


def ease_out_cubic(x: float) -> float:
    """缓动函数easeOutCubic

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - math.pow(1 - x, 3)


def ease_in_out_cubic(x: float) -> float:
    """缓动函数easeInOutCubic

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 4 * x * x * x if x < 0.5 else 1 - math.pow(-2 * x + 2, 3) / 2


def ease_in_quart(x: float) -> float:
    """缓动函数easeInQuart

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return math.pow(x, 4)


def ease_out_quart(x: float) -> float:
    """缓动函数easeOutQuart

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - math.pow(1 - x, 4)


def ease_in_out_quart(x: float) -> float:
    """缓动函数easeInOutQuart

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 8 * x * x * x * x if x < 0.5 else 1 - math.pow(-2 * x + 2, 4) / 2


def ease_in_quint(x: float) -> float:
    """缓动函数easeInQuint

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return math.pow(x, 5)


def ease_out_quint(x: float) -> float:
    """缓动函数easeOutQuint

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - math.pow(1 - x, 5)


def ease_in_out_quint(x: float) -> float:
    """缓动函数easeInOutQuint

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 16 * x * x * x * x * x if x < 0.5 else 1 - math.pow(-2 * x + 2, 5) / 2


def ease_in_expo(x: float) -> float:
    """缓动函数easeInExpo

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 0 if x == 0 else math.pow(2, 10 * x - 10)


def ease_out_expo(x: float) -> float:
    """缓动函数easeOutExpo

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 if x == 1 else 1 - math.pow(2, -10 * x)


def ease_in_out_expo(x: float) -> float:
    """缓动函数easeInOutExpo

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    if x == 0:
        return 0
    if x == 1:
        return 1
    return math.pow(2, 20 * x - 10) / 2 if x < 0.5 else (2 - math.pow(2, -20 * x + 10)) / 2


def ease_in_circ(x: float) -> float:
    """缓动函数easeInCirc

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - math.sqrt(1 - math.pow(x, 2))


def ease_out_circ(x: float) -> float:
    """缓动函数easeOutCirc

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - math.sqrt(1 - math.pow(x, 2))


def ease_in_out_circ(x: float) -> float:
    """缓动函数easeInOutCirc

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return (1 - math.sqrt(1 - math.pow(2 * x, 2))) / 2 if x < 0.5 else (math.sqrt(1 - math.pow(-2 * x + 2, 2)) + 1) / 2


def ease_in_back(x: float) -> float:
    """缓动函数easeInBack

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    c1 = 1.70158
    c3 = c1 + 1
    return c3 * pow(x, 3) - c1 * x * x


def ease_out_back(x: float) -> float:
    """缓动函数easeOutBack

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    c1 = 1.70158
    c3 = c1 + 1

    return 1 + c3 * math.pow(x - 1, 3) + c1 * math.pow(x - 1, 2)


def ease_in_out_back(x: float) -> float:
    """缓动函数easeInOutBack

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    c1 = 1.70158
    c2 = c1 * 1.525

    return (math.pow(2 * x, 2) * ((c2 + 1) * 2 * x - c2)) / 2 if x < 0.5 else (math.pow(2 * x - 2, 2) * (
            (c2 + 1) * (x * 2 - 2) + c2) + 2) / 2


def ease_in_elastic(x: float) -> float:
    """缓动函数easeInElastic

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    c4 = (2 * math.pi) / 3
    if x == 0:
        return 0
    if x == 1:
        return 1
    return -math.pow(2, 10 * x - 10) * math.sin((x * 10 - 10.75) * c4)


def ease_out_elastic(x: float) -> float:
    """缓动函数easeOutElastic

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    c4 = (2 * math.pi) / 3
    if x == 0:
        return 0
    if x == 1:
        return 1
    return math.pow(2, -10 * x) * math.sin((x * 10 - 0.75) * c4) + 1


def ease_in_out_elastic(x: float) -> float:
    """缓动函数easeInOutElastic

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    c5 = c5 = (2 * math.pi) / 4.5
    if x == 0:
        return 0
    if x == 1:
        return 1
    return -(math.pow(2, 20 * x - 10) * math.sin((20 * x - 11.125) * c5)) / 2 if x < 0.5 else (
        (math.pow(2, -20 * x + 10) * math.sin((20 * x - 11.125) * c5)) / 2 + 1
    )


def ease_in_bounce(x: float) -> float:
    """缓动函数easeInBounce

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return 1 - ease_out_bounce(1 - x)


def ease_out_bounce(x: float) -> float:
    """缓动函数easeOutBounce

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    n1 = 7.5625
    d1 = 2.75

    if x < 1 / d1:
        return n1 * x * x
    if x < 2 / d1:
        x -= 1.5 / d1
        return n1 * x * x + 0.75
    if x < 2.5 / d1:
        x -= 2.25 / d1
        return n1 * x * x + 0.9375
    x -= 2.625
    return n1 * x * x + 0.984375


def ease_in_out_bounce(x: float) -> float:
    """缓动函数easeInOutBounce

    Args:
        x: 横轴值, 范围 [0, 1]

    Returns:
        函数纵轴值, 范围 [0, 1]
    """
    return (1 - ease_out_bounce(1 - 2 * x)) / 2 if x < 0.5 else (1 + ease_out_bounce(2 * x - 1)) / 2



import dataclasses
import inspect
import json
import uuid

from dragons96_tools import ioc
from dragons96_tools.ioc.beans.factory import InitializingBean
from dragons96_tools.ioc.config import load_config, cfg


@ioc.Bean  # 默认name会转为小驼峰 randomUuidFunc
def random_uuid_func():
    return uuid.uuid4


@ioc.Bean
def int_bean():
    return 1


@ioc.Bean
class A:
    def __init__(self, uuid_func=ioc.Autowired(value='randomUuidFunc')):
        # 不支持在init中操作uuid_func依赖注入对象
        self._uuid_func = uuid_func
        self._uuid = None

    def __post_construct__(self):
        self._uuid = self._uuid_func()
        print('A.__post_construct__被执行, uuid=', self._uuid)

    def data(self):
        return self._uuid


@ioc.Bean(name='a2', primary=True)
def a():
    return A()


@ioc.Bean
class B(InitializingBean):
    def after_properties_set(self):
        print('B.after_properties_set被执行, a.data() = ', self._a.data())

    def __init__(self, a: A = ioc.Autowired(cls=A)):
        self._a = a

    def data(self):
        return 'B delay {}'.format(self._a.data())


@ioc.Bean
class C:
    a: int
    b: int
    c: str

    def __init__(self, a=ioc.Value('${c.a}'),
                 b=ioc.Value('${c.b}'),
                 c=ioc.Value('${c.c}', default_value=None),
                 ):
        self.a = a
        self.b = b
        self.c = c


def test_ioc():
    # 启动ioc, 扫描当期tests包
    ioc.run(scan_package_names='tests.test_ioc', config_dir='config')
    context = ioc.get_application_context()
    v = context.get_bean_by_name('intBean')
    print(v.get())
    a = context.get_bean_by_name('a')
    print(a.data())
    b = context.get_bean_by_type(B)
    print(b.data())
    v = context.get_bean_by_type(int)
    print(v == 1)
    c = context.get_bean('c')
    print(c)



import abc
from typing import Union, Type, Callable, Any


class BeanFactory(abc.ABC):
    """Bean工厂抽象类"""

    @abc.abstractmethod
    def get_bean(self, name: str = None, required_type: Union[Type, Callable] = None):
        """获取bean方法"""
        raise NotImplemented('未实现该方法')

    @abc.abstractmethod
    def register_bean(self, name: str, cls, primary: bool = False) -> Any:
        """bean注册方法"""
        raise NotImplemented('未实现该方法')

    @abc.abstractmethod
    def init(self):
        """创建bean的初始化方法"""
        raise NotImplemented('未实现该方法')


import inspect

from dragons96_tools.ioc.base import _register_bean


def bean(*args, name: str = None, primary=False):
    """bean装饰器

    Args:
        name: bean name
        primary: 是否默认, 当通过类型获取bean有多个bean时， 若无有primary或存在多个primary的bean将抛出异常, 仅当有一个primary时正常返回
    """

    def wrapper(fc=None):
        # 注册bean
        _register_bean(name=name, cls=fc, primary=primary)
        fc.__bean__ = True
        return fc

    if len(args) == 1 and (inspect.isclass(args[0]) or inspect.isfunction(args[0])):
        return wrapper(args[0])
    return wrapper


Bean = bean



import uuid
from abc import ABC, abstractmethod
from dragons96_tools.models import BaseModel
from contextlib import contextmanager
from typing import Any, TypeVar


class ProxyInfo(BaseModel):
    """代理信息"""
    # 代理ID, 与IP或者上线文关联, 业务自己实现, 例如uuid等
    proxy_id: str
    # 代理信息, 具体由业务决定, 例如http代理需要包含http代理的url(有些还需要账号密码, 加密等参数)
    info: Any


_PROXY_INFO = TypeVar('_PROXY_INFO', bound=ProxyInfo)


class ProxyPool(ABC):
    """代理池抽象封装"""

    def __init__(self):
        self._proxy_map = {}

    def acquire(self, proxy_id: str) -> _PROXY_INFO:
        """获取一个代理信息, 若第一次获取则从池中生成一个代理

        Args:
            proxy_id: 代理ID

        Returns:
            代理信息
        """
        proxy_info = self._proxy_map.get(proxy_id)
        if proxy_info and not self.check_proxy_expired(proxy_id):
            return proxy_info
        proxy_info = self._gen_proxy_info()
        if proxy_info:
            self._proxy_map[proxy_id] = proxy_info
        return proxy_info

    def check_proxy_expired(self, proxy_id: str) -> bool:
        """检查代理是否已过期

        Args:
            proxy_id: 代理ID

        Returns:
            代理过期状态, true=已过期, false=未过期
        """
        proxy_info = self._proxy_map.get(proxy_id)
        if not proxy_info:
            return True
        return self._check_proxy_expired_impl(proxy_info)

    @abstractmethod
    def _check_proxy_expired_impl(self, proxy_info: _PROXY_INFO) -> bool:
        """检查代理是否过期

        Args:
            proxy_info: 代理信息

        Returns:
            代理过期状态, true=已过期, false=未过期
        """
        raise NotImplemented('未实现检查代理是否过期方法')

    def release(self, proxy_id: str):
        """释放一个代理, 将代理归还入池

        Args:
            proxy_id: 代理ID
        """
        if proxy_id in self._proxy_map:
            del self._proxy_map[proxy_id]

    @abstractmethod
    def _gen_proxy_info(self) -> _PROXY_INFO:
        """生成一个代理信息对象

        Returns:
            代理信息
        """
        raise NotImplemented('未实现生成代理方法')


class ProxyManager:
    """代理管理器"""

    def __init__(self, pool: ProxyPool):
        self._pool = pool

    @contextmanager
    def proxy(self) -> _PROXY_INFO:
        """获取一个代理

        使用示例:
            manager = ProxyManager(pool=xxx)
            with manager.proxy() as proxy_info:
                # 业务逻辑
                ...
        """
        proxy_id = str(uuid.uuid4())
        try:
            proxy_info = self._pool.acquire(proxy_id=proxy_id)
            yield proxy_info
        finally:
            self._pool.release(proxy_id=proxy_id)



from dragons96_tools.models import BaseModel
from dragons96_tools.clickhouse.clickhouse_driver import ClickhouseDriverClient
from loguru import logger
from typing import Optional


class TestPythonTblModel(BaseModel):
    id: int
    name: str


def get_client():
    return ClickhouseDriverClient(
        host="localhost",
        port=9000,
        database="test",
        user="root",
        password="123456"
    )


def test_clickhouse_driver():
    client = get_client()
    client.execute('insert into test_python_tbl(`id`, `name`) values(1, \'测试名称\')')
    res = client.execute('select * from test_python_tbl limit 10', modelclass=TestPythonTblModel)
    logger.info(res)



from loguru import logger


def setup(file_name,
          rotation="1 days",
          retention="30 days",
          serialize=True,
          backtrace=True,
          diagnose=False,
          filter=lambda record: True,
          level='INFO'):
    """初始化安装loguru日志配置

    使用方式:
        from loguru import logger

        logger.info('info message')
        logger.error('error message')

    Args:
        file_name: 文件名称
        rotation: 日志滚动周期, 允许值示例如下:
            logger.add("file_1.log", rotation="500 MB")    # Automatically rotate too big file
            logger.add("file_2.log", rotation="12:00")     # New file is created each day at noon
            logger.add("file_3.log", rotation="1 week")    # Once the file is too old, it's rotated
            logger.add("file_X.log", retention="10 days")  # Cleanup after some time
            logger.add("file_Y.log", compression="zip")    # Save some loved space
        retention: 日志文件保留周期, 默认30天
        serialize: 是否将写入文件的数据序列化为json, 默认True. 推荐: True
        backtrace: 是否默认在异常时自动打印堆栈信息, 默认True. 推荐: True
        diagnose: 是否开启代码异常诊断, 可层层递进追踪异常链路, 生产环境建议关闭, 开发环境建议开启, 默认False
        filter: 日志文件的过滤条件, 符合条件的日志记录将写入该文件
        level: 日志级别, 只有超过该级别的日志会记入文件, 支持以下日志级别
            TRACE: 最低级别，用于追踪程序的详细执行流程，通常用于诊断问题
            DEBUG: 用于调试目的的详细信息，例如变量值或者函数调用
            INFO: 提供程序执行中的一般信息，用于表示程序正常运行
            SUCCESS: 表示成功的操作
            WARNING: 表示潜在的问题或者不符合预期的情况，但程序仍能继续执行
            ERROR: 表示错误，但程序仍然可以继续执行
            CRITICAL: 表示严重错误，可能导致程序无法继续执行
    """
    # 添加文件配置
    logger.add(file_name,
               format="{message}",
               backtrace=backtrace,
               diagnose=diagnose,
               serialize=serialize,
               rotation=rotation,
               retention=retention,
               filter=filter,
               level=level)


class PrefixLogger:
    """前缀日志工具, 所有日志内容都将带上[prefix]前缀"""

    def __init__(self, prefix='', _logger=None):
        self._prefix = prefix
        if _logger:
            self._logger = _logger
        else:
            self._logger = logger

    def trace(self, __message, *args, **kwargs):
        return self.log("TRACE", __message, *args, **kwargs)

    def debug(self, __message, *args, **kwargs):
        return self.log("DEBUG", __message, *args, **kwargs)

    def info(self, __message, *args, **kwargs):
        return self.log("INFO", __message, *args, **kwargs)

    def warning(self, __message, *args, **kwargs):
        return self.log("WARNING", __message, *args, **kwargs)

    def error(self, __message, *args, **kwargs):
        return self.log("ERROR", __message, *args, **kwargs)

    def success(self, __message, *args, **kwargs):
        return self.log("SUCCESS", __message, *args, **kwargs)

    def critical(self, __message, *args, **kwargs):
        return self.log("CRITICAL", __message, *args, **kwargs)

    def exception(self, __message, *args, **kwargs):
        if self._prefix:
            return self._logger.exception(f"[{self._prefix}]{__message}", *args, **kwargs)
        return self._logger.exception(__message, *args, **kwargs)

    def log(self, __level, __message, *args, **kwargs):
        if self._prefix:
            return self._logger.log(__level, f"[{self._prefix}]{__message}", *args, **kwargs)
        return self._logger.log(__level, __message, *args, **kwargs)



from dragons96_tools.ioc.beans.proxy import DelayAutowiredClassBeanProxy, DelayConfigAutowiredClassBeanProxy
from dragons96_tools.ioc.constants import required


class Autowired:
    """依赖自动注入"""

    def __new__(_, value: str = None, *, cls=None, required: bool = True):
        """注入对象

        Args:
            value: bean name
            required: 是否必须, 为true找不到注入对象会报错
        """
        return DelayAutowiredClassBeanProxy(name=value, obj=cls, required=required)


class Value:
    """配置自动注入"""
    _cache = {}

    def __new__(_, value: str, *args, cls=None, default_value=required):
        """参数注入

        Args:
            value: 配置名称, 多级使用.分隔, 示例: xxx.xxx
            cls: value读取数据的类型, 如果为None则返回环境变量中的值
            default_value: 默认值, 若不传则表示必须有配置值
        """
        if not value:
            raise ValueError('value配置名称不能为空')
        return DelayConfigAutowiredClassBeanProxy(name=value, obj=cls, default_value=default_value)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as readme_file:
    readme = readme_file.read()

requirements = []

with open('requirements.txt', encoding='utf-8') as f:
    for require in f.readlines():
        require = require.strip()
        if require:
            requirements.append(require)

test_requirements = ['pytest>=6.2.4', ]

setup(
    author="dragons96_tools",
    author_email='521274311@qq.com',
    python_requires='>=3.9',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.9',
    ],
    description="dragons96 个人开发python工具包",
    install_requires=requirements,
    extras_require={
        'mysql': ['pymysql>=1.1.0'],
        'orm': ['sqlalchemy>=2.0.27'],
        'impala': ['impyla>=0.19.0'],
        'clickhouse': ['clickhouse-driver>=0.2.7'],
        'orm_clickhouse': ['sqlalchemy>=2.0.27', 'clickhouse-sqlalchemy'],
        'hive': ['pyhive', 'thrift', 'thrift-sasl'],
        'fastapi': ['fastapi>=0.110.0', 'uvicorn[standard]>=0.29.0'],
        'aes': ['pycryptodome>=3.20.0'],
        'all': ['pymysql>=1.1.0', 'sqlalchemy>=2.0.27', 'impyla>=0.19.0', 'clickhouse-driver>=0.2.7',
                'fastapi>=0.110.0', 'uvicorn[standard]>=0.29.0', 'pycryptodome>=3.20.0', 'pyhive', 'thrift',
                'thrift-sasl', 'pycryptodome>=3.20.0'],
    },
    license="MIT license",
    long_description=readme,
    include_package_data=True,
    keywords='dragons96_tools',
    name='dragons96_tools',
    packages=find_packages(include=['dragons96_tools', 'dragons96_tools.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://gitee.com/dragons96/py_dragons96_tools',
    version='0.0.46',
    zip_safe=False,
)



import os
from enum import Enum
from loguru import logger as log


class EnvEnum(Enum):
    # 开发环境
    dev = 'development'
    # 测试环境
    test = 'test'
    # 生产环境
    pro = 'production'

    def is_dev(self):
        if self.value == 'development':
            return True
        return False

    def is_test(self):
        if self.value == 'test':
            return True
        return False

    def is_pro(self):
        if self.value == 'production':
            return True
        return False


_env = None


def get_env() -> EnvEnum:
    """获取系统当前环境, 仅支持EnvEnum定义的环境名称

    Returns:
        EnvEnum 环境枚举对象
    """
    global _env
    if _env:
        return _env
    os_env = os.getenv('ENV', 'dev').lower()
    for env in EnvEnum:
        if env.name == os_env:
            _env = env
            break
    if not _env:
        _env = EnvEnum.dev
        log.warning('未找到设置的系统环境类型[{}], 当前使用默认环境类型[{}]'.format(os_env, _env.value))
    log.info('当前项目环境: [{}]'.format(_env.name))
    return _env
